﻿-- Author      : RisM
-- Create Date : 6/28/2009 4:02:20 PM

local SpeakinSpell = LibStub("AceAddon-3.0"):GetAddon("SpeakinSpell")
local L	= LibStub("AceLocale-3.0"):GetLocale("SpeakinSpell", false)

SpeakinSpell:PrintLoading("wowevents.lua")

-------------------------------------------------------------------------------
-- WOW GAME EVENT HANDLERS
-------------------------------------------------------------------------------


function SpeakinSpell:RegisterAllEvents()
	-- register for spellcasting events
	-- which is our hook too know when to speak for a spell
	-- among other things
	
	-- startup and loading events
	self:RegisterEvent("VARIABLES_LOADED")
	
	-- combat events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	--self:RegisterEvent("UNIT_AURA") --see UNIT_AURA below

	-- enter and exit combat
	-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
	--		Those 2 events are auto-attack on/off notifications
	--		To detect when the combat flag goes on and off from getting/losing aggro
	--		which is what we generally mean by "entering combat" and "exiting combat"
	--		we must check for Regen enabled/disabled, which is our only valid notification
	--		see also: http://www.wowwiki.com/Events/Combat
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	
	-- change zones --
	--self:RegisterEvent("MINIMAP_ZONE_CHANGED") --for subzone changes - does not appear to be needed
	self:RegisterEvent("ZONE_CHANGED_INDOORS") --for subzone changes inside an instance
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA") --for large scale zone changes
	self:RegisterEvent("ZONE_CHANGED") --covers both small and large scale zone changes?
	
	-- chat events, such as receiving a whisper
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_GUILD")
	self:RegisterEvent("CHAT_MSG_PARTY")

	-- detecting /follow
	self:RegisterEvent("AUTOFOLLOW_BEGIN")
	self:RegisterEvent("AUTOFOLLOW_END")
	
	-- Achievements
	self:RegisterEvent("ACHIEVEMENT_EARNED")
	self:RegisterEvent("CHAT_MSG_ACHIEVEMENT")
	self:RegisterEvent("CHAT_MSG_GUILD_ACHIEVEMENT")

	-- NPC interaction
	self:RegisterEvent("GOSSIP_SHOW")
	self:RegisterEvent("BARBER_SHOP_OPEN")
	self:RegisterEvent("BARBER_SHOP_CLOSE")
	self:RegisterEvent("MAIL_SHOW")
	self:RegisterEvent("MERCHANT_SHOW")
	self:RegisterEvent("QUEST_GREETING")
	self:RegisterEvent("TAXIMAP_OPENED")
	self:RegisterEvent("TRAINER_SHOW")
	
	-- more miscellaneous events
	self:RegisterEvent("PLAYER_LEVEL_UP")
	self:RegisterEvent("RESURRECT_REQUEST")
	self:RegisterEvent("TRADE_SHOW")
end


---------------------------------------------------------------------------
-- STARTUP AND LOADING EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:VARIABLES_LOADED()
	self:OnVariablesLoaded() -- redirected to be near OnInitialize
end



---------------------------------------------------------------------------
-- COMBAT EVENTS
---------------------------------------------------------------------------


function SpeakinSpell:UNIT_AURA(_, unitId) --weak, Blizzard...
	--TODO: Blizzard added a bug in 3.3.0 where some buffs don't generate SPELL_AURA_APPLIED anymore
	--		the only alternative at this point is UNIT_AURA, but this only gives the unitId
	--		a loop and a history list are required to determine which is the new buff or debuff
	--		based on UnitAura API.  Some other dev in that area might be nice
	--		to support randomized asking for buffs, i.e. the buffs you still need, the buffs people in group still need
	--		it's non-trivial.
end


function SpeakinSpell:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local funcname = "SPELL_AURA_APPLIED"
	-- we're only concerned with buffs we receive
	if dstName ~= UnitName("player") then
		--self:DebugMsg(funcname, "not cast on you")
		return
	end
	
	-- extra parameters provided with "SPELL_" prefix
	local spellId, spellName, spellSchool = select(1, ...)
	-- extra parameters provided with "_AURA_APPLIED" suffix (same for _AURA_REFRESH)
	local auraType = select(4, ...)

	self:DebugMsg(funcname, "( srcName, dstName, auraType, spellName )")
	self:DebugMsgDumpString("srcName",srcName)
	self:DebugMsgDumpString("dstName",dstName)
	self:DebugMsgDumpString("auraType",auraType)
	self:DebugMsgDumpString("spellName",spellName)
	
	local DetectedEventStub = {
		-- event descriptors
		name = spellName,
		type = "SPELL_AURA_APPLIED_"..tostring(auraType),
		--rank = "", -- no rank known
		-- event-specific data for substitutions
		caster = srcName,
		target = dstName,
		spellid = spellId,
	}
	
	-- buffs cast on me by someone other than me are treated differently than buffs I cast on myself
	if srcName ~= UnitName("player") then
		self:DebugMsg(funcname, "not cast by you")
		DetectedEventStub.type = tostring(DetectedEventStub.type).."_FOREIGN"
	end
	
	-- process the spell
	-- this function is shared with the UNIT_SPELLCAST_SENT event handler
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:MakeDamageType(amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	-- create a damage type string containing all applicable: resisted, blocked, absorbed, critical, glancing, crushing
	-- this is used in the event name, and thus also the EventTable key
	local DamageType = ""
	local function AppendDamageType( condition, label )
		if condition then
			if DamageType == "" then
				DamageType = L[label]
			else
				DamageType = DamageType .. L[", "] .. L[label]
			end
		end
	end
	
	-- all applicable damage/hit/miss types will be appended in this order
	AppendDamageType(crushing,	"Crushing")
	AppendDamageType(critical,	"Critical")
	AppendDamageType(resisted,	"Resisted")
	AppendDamageType(blocked,	"Blocked")
	AppendDamageType(absorbed,	"Absorbed")
	AppendDamageType(glancing,	"Glancing")
	AppendDamageType(overkill and overkill > 0, "Killing Blow")
	-- if none of the other conditions apply, then it's a normal hit
	if DamageType == "" then
		DamageType = L["Hit"]
	end
	
	return DamageType
end


function SpeakinSpell:SPELL_DAMAGE(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
	local funcname = "SPELL_DAMAGE"
	--eventtype will always be "SPELL_DAMAGE"
	
	-- we only care about damage done to me, or done by me
	local nameformat = L["I caused Yellow Damage (%s)"]
	if srcName ~= UnitName("player") then
		if dstName ~= UnitName("player") then
			--self:DebugMsg(funcname, "you must neither the caster nor the target")
			return
		end
		nameformat = L["I received PvP Yellow Damage (%s)"]
	end

	-- extra parameters provided with "SPELL_" prefix
	local spellId, spellName, spellSchool = select(1, ...)
	-- extra parameters provided with "_DAMAGE" suffix
	local amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(4, ...)
	
	-- create a damage type string containing all applicable: resisted, blocked, absorbed, critical, glancing, crushing
	-- this is used in the event name, and thus also the EventTable key
	local DamageType = self:MakeDamageType(amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	
	local DetectedEventStub = {
		type = "COMBAT",
		
		-- name = eventname = we don't know which event name to use yet, but we'll build it slowly
		-- we're going to build an event name (and key) of the form
		-- Spell Damage - resisted, blocked, absorbed, critical, glancing, crushing, killing blow
		-- for all that apply
		name		= string.format( nameformat, DamageType),
		eventname	= string.format( nameformat, DamageType),
		
		-- replace the default spellname = eventname = name logic, to provide info about the actual spell
		spellid = spellId,
		spellname = spellName,
		
		-- spell/ability was cast by source on dest
		caster = srcName,
		target = dstName,
		
		-- event-specific substitions
		damage		= amount or 0,
		overkill	= overkill or 0,
		school		= self:DamageSchoolCodeNumberToString(school), --spellSchool is the same code number
		damagetype	= DamageType,
	}
	
	-- override the display link format used by the Report Detected Speech Events diagnostic feature, to achieve this results:
	-- self:Print( "Combat Event: Yellow Damage (Hit): [Seal of Righteousness]" )
	DetectedEventStub.displaylink = self:FormatSubs( "<eventtypeprefix><eventname>: <spelllink>", DetectedEventStub)
	
	self:OnSpeechEvent( DetectedEventStub )
end



function SpeakinSpell:SWING_DAMAGE(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
	local funcname = "SWING_DAMAGE"
	--eventtype will always be "SWING_DAMAGE"
	
	-- we only care about damage done to me, or done by me
	local nameformat = L["I caused White Damage (%s)"]
	if srcName ~= UnitName("player") then
		if dstName ~= UnitName("player") then
			--self:DebugMsg(funcname, "you must neither the caster nor the target")
			return
		end
		nameformat = L["I received PvP White Damage (%s)"]
	end

	-- extra parameters provided with "SWING_" prefix
	--none
	-- extra parameters provided with "_DAMAGE" suffix
	local amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing = select(1, ...)
	
	-- create a damage type string containing all applicable: resisted, blocked, absorbed, critical, glancing, crushing
	-- this is used in the event name, and thus also the EventTable key
	local DamageType = self:MakeDamageType(amount, overkill, school, resisted, blocked, absorbed, critical, glancing, crushing)
	
	local DetectedEventStub = {
		type = "COMBAT",
		
		-- name = eventname = we don't know which event name to use yet, but we'll build it slowly
		-- we're going to build an event name (and key) of the form
		-- Spell Damage - resisted, blocked, absorbed, critical, glancing, crushing, killing blow
		-- for all that apply
		name		= string.format( nameformat, DamageType),
		eventname	= string.format( nameformat, DamageType),
		
		-- replace the default spellname = eventname = name logic, to provide info about the actual spell
		--spellid = spellId, -- no spellId for SWING_ events
		--spelllink = GetSpellLink(spellId), -- no spellId for SWING_ events
		spellname = L["Melee Swing"],
		
		-- spell/ability was cast by source on dest
		caster = srcName,
		target = dstName,
		
		-- event-specific substitions
		damage		= amount or 0,
		overkill	= overkill or 0,
		school		= self:DamageSchoolCodeNumberToString(school),
		damagetype	= DamageType,
	}
	
	-- override the display link format used by the Report Detected Speech Events diagnostic feature, to achieve this results:
	-- self:Print( "Combat Event: White Damage (Critical, Killing Blow): [Swing]" )
	DetectedEventStub.displaylink = self:FormatSubs( "<eventtypeprefix><eventname>: <spelllink>", DetectedEventStub)
	
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:COMBAT_LOG_EVENT_UNFILTERED(_,timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	--self:DebugMsg("COMBAT_LOG_EVENT_UNFILTERED",tostring(eventtype))
	
	-- This is like RegisterEvent: set up a table mapping the eventtype to a function
	SpeakinSpell.RuntimeData.CombatLogEvents = {
		["SPELL_AURA_APPLIED"] = function(  timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
			-- buff gains
			SpeakinSpell:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
		end,
		["SPELL_AURA_REFRESH"] = function(  timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
			-- buff timer refreshed (Recast on you without wearing off all the way first)
			-- NOTE: the function is called SPELL_AURA_APPLIED because it's shared, but still pass eventtype=SPELL_AURA_REFRESH
			SpeakinSpell:SPELL_AURA_APPLIED(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
		end,
		["SPELL_DAMAGE"] = function(  timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
			-- any spell or other special ability landed a hit (or crit) to deal damage
			SpeakinSpell:SPELL_DAMAGE(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
		end,
		["SWING_DAMAGE"] = function(  timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
			-- any spell or other special ability landed a hit (or crit) to deal damage
			SpeakinSpell:SWING_DAMAGE(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...) 
		end,
	}
	local func = SpeakinSpell.RuntimeData.CombatLogEvents[eventtype]
	if func then
		func(timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	end
end


function SpeakinSpell:UNIT_SPELLCAST_StandardHandler(event, caster, target, spellname, spellrank)
	local funcname = event

	self:DebugMsg(funcname,"(caster, spellname, spellrank, target)")
	--self:DebugMsgDumpString("event",event) -- the event name is already the funcname
	self:DebugMsgDumpString("caster",caster)
	self:DebugMsgDumpString("spellname",spellname)
	self:DebugMsgDumpString("spellrank",spellrank)
	self:DebugMsgDumpString("target",target)
	
	-- only speak for spells cast by the player, not anyone else
	if caster == "player" then
		caster = UnitName("player")
	else
		self:DebugMsg(funcname,"caster is not player")
		return
	end
	
	-- process the spell
	local DetectedEventStub = {
		-- event descriptors
		type = event,
		name = spellname,
		rank = spellrank,
		-- event-specific data for substitutions
		caster = caster,
		target = target,
		spellid = self:GetSpellID(spellname, spellrank),
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:UNIT_SPELLCAST_SENT(event, caster, spellname, spellrank, target)
	--subsequent UNIT_SPELLCAST notifications don't include this target info
	-- for some spells like arcane missiles, this target applies
	-- for others like summoning stones, there is no preceding UNIT_SPELLCAST_SENT, it just goes straight to channeling
	-- so this should not be used lightly as a replacement for <target> in all cases, but it does become useful to know
	-- this supports <lasttarget> and 
	self.RuntimeData.LastSpellcastSentTarget = target
	self.RuntimeData.LastInterrupt = nil -- new spell cast can be interrupted again - see UNIT_SPELLCAST_INTERRUPTED
	self:UNIT_SPELLCAST_StandardHandler(event, caster, target, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_INTERRUPTED(event, caster, spellname, spellrank)
	-- these messages can be redundant for a single interrupt notified 3 times
	if self.RuntimeData.LastInterrupt == spellname then
		self:DebugMsg(nil,"suppressed redundant UNIT_SPELLCAST_INTERRUPTED for "..tostring(spellname))
		return
	end
	self.RuntimeData.LastInterrupt = spellname
	-- REVIEW: is it really appropriate to assume that the unknown target of this interrupted spell is the self.RuntimeData.LastSpellcastSentTarget?
	self:UNIT_SPELLCAST_StandardHandler(event, caster, self.RuntimeData.LastSpellcastSentTarget, spellname, spellrank)
end

-- REVIEW: is it really appropriate to assume that the unknown target of this failed spell 
--		is the self.RuntimeData.LastSpellcastSentTarget?
--		not all events are always preceded by UNIT_SPELLCAST_SENT -- cast in point: summoning stone effect
function SpeakinSpell:UNIT_SPELLCAST_FAILED(event, caster, --[[target unknown, --]]                  spellname, spellrank)
	self:UNIT_SPELLCAST_StandardHandler    (event, caster, self.RuntimeData.LastSpellcastSentTarget, spellname, spellrank)
	--self.RuntimeData.LastSpellcastSentTarget = nil
end


-- NOTE: the target is unknown for UNIT_SPELLCAST_CHANNEL_START / STOP
--		The target of the last UNIT_SPELLCAST_SENT (<lasttarget>) is valid for some spells like arcane missiles
--		However, in the case of the summoning stone effect, and maybe other spells, 
--		the channeling event is not preceded by UNIT_SPELLCAST_SENT, so that value is incorrect
function SpeakinSpell:UNIT_SPELLCAST_CHANNEL_START  (event, caster,      spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, nil, spellname, spellrank)
end

function SpeakinSpell:UNIT_SPELLCAST_CHANNEL_STOP   (event, caster,      spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, nil, spellname, spellrank)
end

-- REVIEW: is it really appropriate to assume that the unknown target of this stopped spell 
--		is the self.RuntimeData.LastSpellcastSentTarget?
--		not all events are always preceded by UNIT_SPELLCAST_SENT -- cast in point: summoning stone effect
function SpeakinSpell:UNIT_SPELLCAST_STOP           (event, caster, --[[target unknown, --]]                  spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, self.RuntimeData.LastSpellcastSentTarget, spellname, spellrank)
end

-- REVIEW: is it really appropriate to assume that the unknown target of this stopped spell 
--		is the self.RuntimeData.LastSpellcastSentTarget?
--		not all events are always preceded by UNIT_SPELLCAST_SENT -- cast in point: summoning stone effect
function SpeakinSpell:UNIT_SPELLCAST_SUCCEEDED      (event, caster, --[[target unknown, --]]                  spellname, spellrank)
	             self:UNIT_SPELLCAST_StandardHandler(event, caster, self.RuntimeData.LastSpellcastSentTarget, spellname, spellrank)
end


---------------------------------------------------------------------------
-- ENTERING AND EXITING COMBAT
---------------------------------------------------------------------------
-- NOTE: PLAYER_ENTER_COMBAT and PLAYER_LEAVE_COMBAT are not what they sound like
--		Those 2 events are auto-attack on/off notifications
--		To detect when the combat flag goes on and off from getting/losing aggro
--		which is what we generally mean by "entering combat" and "exiting combat"
--		we must check for Regen enabled/disabled, which is our only valid notification
--		see also: http://www.wowwiki.com/Events/Combat


function SpeakinSpell:PLAYER_REGEN_DISABLED()
	local funcname = "PLAYER_REGEN_DISABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = true
	self:ResetOncePerCombatFlags()

	-- Signal the Entering Combat event
	self:DebugMsg(funcname, "Signal System Event: Entering Combat")
	local DetectedEventStub = {
		-- event descriptors
		name = L["Entering Combat"],
		type = "COMBAT",
		-- event-specific data for substitutions
		-- None
	}
	self:OnSpeechEvent( DetectedEventStub ) 
end


function SpeakinSpell:PLAYER_REGEN_ENABLED()
	local funcname = "PLAYER_REGEN_ENABLED"

	-- update combat status flags for "limit once per combat" feature
	self.RuntimeData.InCombat = false
	self:ResetOncePerCombatFlags()
	
	-- Signal the Exiting Combat event
	self:DebugMsg(funcname, "Signal System Event: Exiting Combat")
	local DetectedEventStub = {
		-- event descriptors
		name = L["Exiting Combat"],
		type = "COMBAT",
		-- event-specific data for substitutions
		-- None
	}
	self:OnSpeechEvent( DetectedEventStub ) 
end


---------------------------------------------------------------------------
-- MOVING TO A DIFFERENT ZONE OR REGION
---------------------------------------------------------------------------


function SpeakinSpell:OnZoneChange(minoronly)
	local funcname = "OnZoneChange"

	-- Signal the zone change event
	self:DebugMsg(funcname, "Signal System Event: Zone Changed, minoronly="..tostring(minoronly))
	local DetectedEventStub = {
		-- event descriptors
		name = L["Changed Zone"],
		rank = "",
		type = "EVENT",
		-- treat these special events as cast by the player on himself
		caster = UnitName("player"),
		target = UnitName("player"),
		-- event-specific data for substitutions
		-- None
	}
	
	if minoronly then
		DetectedEventStub.name = L["Changed Sub-Zone"]
	end
	
	self:OnSpeechEvent( DetectedEventStub ) 
end


--for subzone changes -- does not appear to be needed
--[[
function SpeakinSpell:MINIMAP_ZONE_CHANGED()
	local funcname = "MINIMAP_ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end
--]]

--for subzone changes inside an instance
function SpeakinSpell:ZONE_CHANGED_INDOORS()
	local funcname = "ZONE_CHANGED_INDOORS"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end

--for large scale zone changes
function SpeakinSpell:ZONE_CHANGED_NEW_AREA()
	local funcname = "ZONE_CHANGED_NEW_AREA"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(false)
end

--covers both small and large scale zone changes?
function SpeakinSpell:ZONE_CHANGED()
	local funcname = "ZONE_CHANGED"
	self:DebugMsg(funcname, "entry")
	
	self:OnZoneChange(true)
end


---------------------------------------------------------------------------
-- PARSE CHAT MESSAGES
---------------------------------------------------------------------------


--[[
"CHAT_MSG_WHISPER"
	Category: Communication
  	

Fired when a whisper is received from another player.

The rest of the arguments appear to be nil

arg1 
    Message received 
arg2 
    Author 
arg3 
    Language (or nil if universal, like messages from GM) (always seems to be an empty string; argument may have been kicked because whispering in non-standard language doesn't seem to be possible [any more?]) 
arg6 
    status (like "DND" or "GM") 
arg7 
    (number) message id (for reporting spam purposes?) (default: 0) 
arg8 
    (number) unknown (default: 0) 
--]]
function SpeakinSpell:CHAT_MSG_WHISPER(event, msg, author, language, status, id, ...)
	-- Chat Event: Whispered While In-Combat
	if	self.RuntimeData.InCombat		and 
		type(author) == "string"		and
		author ~= UnitName("player")	then

		local DetectedEventStub = {
			type = "CHAT",
			name = L["Whispered While In-Combat"],
			caster = author, -- the name of the player who whispered to you
			target = UnitName("player"), -- the target of the whisper is you, the player
			-- event-specific substitution
			text = msg -- the contents of the message that was whispered to you
		}
		self:OnSpeechEvent( DetectedEventStub )
	end
	
	-- TODOFUTURE: more whisper-response based features could go here
	--	allow other people to trigger /ss commands by whispering to you, similar to auctioneer's whisper activated price lookup feature
	--	auto-advertise in response to "what addon is that?"
end


--[[
"CHAT_MSG_GUILD"
	Category: Communication,Guild
  	
Fired when a message is sent or received in the Guild channel.

arg1 
    Message that was sent 
arg2 
    Author 
arg3 
    Language that the message was sent in 
--]]
function SpeakinSpell:CHAT_MSG_GUILD(event, ...)
	-- "Chat Event: A Guild Member said 'ding'"
	local message = string.lower( tostring(arg1) )
	local ndx = string.find( message, L["ding"] )
	if ndx and ndx >= 0 then
		local DetectedEventStub = {
			type = "CHAT",
			name = L[ [[A guild member said "ding"]] ],
			caster = arg2,
			target = SpeakinSpell:GetGuildName(),
		}
		self:OnSpeechEvent( DetectedEventStub )
	end
end


function SpeakinSpell:CHAT_MSG_PARTY(event, ...)
	-- "Chat Event: A Party Member said 'ding'"
	local message = string.lower( tostring(arg1) )
	local ndx = string.find( message, L["ding"] )
	if ndx and ndx >= 0 then
		local DetectedEventStub = {
			type = "CHAT",
			name = L[ [[A party member said "ding"]] ],
			caster = arg2,
			--target = SpeakinSpell:GetGuildName(), -- assume default target logic
		}
		self:OnSpeechEvent( DetectedEventStub )
	end
end


---------------------------------------------------------------------------
-- ANNOUNCING /FOLLOW
---------------------------------------------------------------------------

function SpeakinSpell:AUTOFOLLOW_BEGIN(event, unit)
	self.RuntimeData.AutoFollowTarget = unit -- saved for AUTOFOLLOW_END
	local DetectedEventStub = {
		type = "EVENT",
		name = L["Begin /follow"],
		target = unit,
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:AUTOFOLLOW_END(event, ...)
	local DetectedEventStub = {
		type = "EVENT",
		name = L["End /follow"],
		target = self.RuntimeData.AutoFollowTarget,
	}
	self:OnSpeechEvent( DetectedEventStub )
end


---------------------------------------------------------------------------
-- ACHIEVEMENTS
---------------------------------------------------------------------------

function SpeakinSpell:ACHIEVEMENT_EARNED(event, AchievementID)
	local IDNumber, Name, Points, Completed, Month, Day, Year, Description, Flags, Image, RewardText = GetAchievementInfo( AchievementID )
	local DetectedEventStub = {
		type = "ACHIEVEMENT",
		name = L["me"], -- DisplayName = "Achievement Earned by <name>"
		-- NOTE: we don't want the name of this achievement to be in the type or name of this stub, defined above, 
		--		because we want ALL differently-named achievements to share the same speech event settings
		--		so we'll use an event-specific custom data name to support substitution the name of this achievement in the speech
		achievement = Name,
		-- and the achievement link also relays the achievement name, via the standard <spelllink> substitution
		spelllink = GetAchievementLink( AchievementID ),
		-- and standard meaning for target and caster OF THE EVENT
		target = UnitName("player"),
		caster = UnitName("player"),
		-- and let's include (most of) the rest of the achievement info for substitutions, why not?
		points = Points,
		desc = Description,
		reward = RewardText,
	}
	self:OnSpeechEvent( DetectedEventStub )
end


--[[
from wowwiki.com, plus added comments to fill in missing details

"CHAT_MSG_ACHIEVEMENT"
	Category: Communication,Guild,Achievements
  	

Fired when a player in your vicinity completes an achievement.

arg1 (ChatMessage)
    The full body of the broadcast message. ("%s has earched the achievement [clickable link]")
arg2, arg5 (EarnedBy)
    The name of player who has just completed the achievement. 
arg7, arg8 
    Some integer. (Ris: the achievement ID???)
--]]
function SpeakinSpell:CHAT_MSG_ACHIEVEMENT(event, ChatMessage, EarnedBy, ...)
	local ndx = string.find(ChatMessage, "|") -- find the clickable link in the achievement message
	local achievement = string.sub(ChatMessage, ndx, string.len(ChatMessage))
	local DetectedEventStub = {
		type = "ACHIEVEMENT",
		name = L["Someone Nearby"], -- DisplayName = "Achievement Earned by <name>"
		-- NOTE: we don't want the name of this achievement to be in the type or name of this stub, defined above, 
		--		because we want ALL differently-named achievements to share the same speech event settings
		--		so we'll use an event-specific custom data name to support substitution the name of this achievement in the speech
		--TODOFUTURE: ? parse this for the achievement name or link instead of the full chat message text ?
		--	for now, providing <achievement> and <desc> as both the complete message, for similarity to ACHIEVEMENT_EARNED
		achievement = achievement, 
		desc = achievement,
		spelllink = achievement,
		-- and standard meaning for target and caster OF THE EVENT (the player who earned the achievement)
		target = EarnedBy,
		caster = EarnedBy,
	}
	self:OnSpeechEvent( DetectedEventStub )
end


--[[
"CHAT_MSG_GUILD_ACHIEVEMENT" (same as CHAT_MSG_ACHIEVEMENT)
	Category: Communication,Guild,Achievements
  	
Fired when a guild member completes an achievement.

arg1 
    The full body of the broadcast message. 
arg2, arg5 
    The name of player who has just completed the achievement. 
arg7, arg8, arg11 
    Some integer that (but not the achievement ID, or the total number of achievement points for the player; this seems to increment if two consecutive achievements are posted (needs to be verified)). 
--]]
function SpeakinSpell:CHAT_MSG_GUILD_ACHIEVEMENT(event, ChatMessage, EarnedBy, ...)
	local ndx = string.find(ChatMessage, "|") -- find the clickable link in the achievement message
	local achievement = string.sub(ChatMessage, ndx, string.len(ChatMessage))
	local DetectedEventStub = {
		type = "ACHIEVEMENT",
		name = L["a Guild Member"], -- DisplayName = "Achievement Earned by <name>"
		-- NOTE: we don't want the name of this achievement to be in the type or name of this stub, defined above, 
		--		because we want ALL differently-named achievements to share the same speech event settings
		--		so we'll use an event-specific custom data name to support substitution the name of this achievement in the speech
		--TODOFUTURE: ? parse this for the achievement name or link instead of the full chat message text ?
		--	for now, providing <achievement> and <desc> as both the complete message, for similarity to ACHIEVEMENT_EARNED
		achievement = achievement, 
		desc = achievement,
		spelllink = achievement,
		-- and standard meaning for target and caster OF THE EVENT (the player who earned the achievement)
		target = EarnedBy,
		caster = EarnedBy,
	}
	self:OnSpeechEvent( DetectedEventStub )
end


---------------------------------------------------------------------------
-- NPC INTERACTION
---------------------------------------------------------------------------


function SpeakinSpell:GOSSIP_SHOW()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Open Gossip Window"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:BARBER_SHOP_OPEN()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Enter Barber Chair"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:BARBER_SHOP_CLOSE()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Exit Barber Chair"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:MAIL_SHOW()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Open Mailbox"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:MERCHANT_SHOW()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Talk to Vendor"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:QUEST_GREETING()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Talk to Quest-Giver"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:TAXIMAP_OPENED()
	local DetectedEventStub = {
		type = "NPC",
		name = L["Talk to Flight Master"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:TRAINER_SHOW() -- fired for any class and profession trainers
	local DetectedEventStub = {
		type = "NPC",
		name = L["Talk to Trainer"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


---------------------------------------------------------------------------
-- MORE MISC. EVENTS
---------------------------------------------------------------------------

--[[
"PLAYER_LEVEL_UP"
	Category: Player
  	

Fired when a player levels up.

arg1 
    New player level. Note that UnitLevel("player") will most likely return an incorrect value when called in this event handler or shortly after, so use this value. 
arg2 
    Hit points gained from leveling. 
arg3 
    Mana points gained from leveling. 
arg4 
    Talent points gained from leveling. Should always be 1 unless the player is between levels 1 to 9. 
arg5 - arg9 
    Attribute score increases from leveling. Strength (5) / Agility (6) / Stamina (7) / Intellect (8) / Spirit (9). 
--]]
function SpeakinSpell:PLAYER_LEVEL_UP(event, ...)
	local DetectedEventStub = {
		type = "EVENT",
		name = L["Level Up"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end


--[[
"RESURRECT_REQUEST"
	Category: Death
  	
Fired when another player resurrects you

arg1 
    player name 
--]]
function SpeakinSpell:RESURRECT_REQUEST(event, caster)
	local DetectedEventStub = {
		type = "EVENT",
		name = L["a player sent me a rez"],
		caster = caster,
		target = UnitName("player"),
	}
	self:OnSpeechEvent( DetectedEventStub )
end


function SpeakinSpell:TRADE_SHOW()
	local DetectedEventStub = {
		type = "EVENT",
		name = L["Open Trade Window"],
	}
	self:OnSpeechEvent( DetectedEventStub )
end